@@ -18,7 +18,9 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #---------------------------------------------------------------------------
-from base_module import ranaModule
+from modules.base_module import ranaModule
+
+import os
 import sys
 import math
 import re
@@ -27,9 +29,16 @@ import traceback
 import unicodedata
 import way
 from time import clock
+import time
 
 DIRECTIONS_FILTER_CSV_PATH = 'data/directions_filter.csv'
 
+ROUTING_SUCCESS = 0
+ROUTING_NO_DATA = 1 # failed to load routing data
+ROUTING_LOAD_FAILED = 2 # failed to load routing data
+ROUTING_LOOKUP_FAILED = 3 # failed to locate nearest way/edge
+ROUTING_ROUTE_FAILED = 4 # failed to compute route
+
 def getModule(m,d,i):
   return(route(m,d,i))
 
@@ -322,10 +331,15 @@ class route(ranaModule):
 
     provider = self.get('routingProvider', "GoogleDirections")
     if provider == "Monav":
+      sentTimestamp = time.time()
+      print('routing: using Monav as routing provider')
       waypoints = [(fromLat, fromLon), (toLat, toLon)]
-      route = self.getMonavRoute(waypoints)
+      result = self.getMonavRoute(waypoints)
       #TODO: asynchronous processing & error notifications
-      self._handleResults("MonavRoute", route)
+      # as monav is VERY fast for routing, the routing might still get done
+      # asynchronously, but the work-in-progress overlay might show up
+      # only once the search takes longer than say 2 seconds
+      self._handleResults("MonavRoute", (result, waypoints[0], waypoints[-1], sentTimestamp) )
 
     else: # use Google Directions as fallback
       online = self.m.get('onlineServices', None)
@@ -333,25 +347,74 @@ class route(ranaModule):
         online.googleDirectionsLLAsync((fromLat, fromLon), (toLat, toLon), self._handleResults, "onlineRoute")
 
   def getMonavRoute(self, waypoints):
-    monavDataFolder = '' #TODO: implement this
-    if monavDataFolder:
+
+    mode = self.get('mode', 'car')
+    # get mode based sub-folder
+    # TODO: handle not all mode folders being available
+    # (eq. user only downloading routing data for cars)
+    modeFolders = {
+      'cycle':'routing_bike',
+      'walk':'routing_pedestrian',
+      'car':'routing_car'
+    }
+    subFolder = modeFolders.get(mode, 'routing_car')
+    try:
+      dataPacks = self.getAvailableMonavDataPacks()
+    except Exception, e:
+      print("route: can't list Monav data directory")
+      print(e)
+      dataPacks = []
+
+    if dataPacks:
+      # TODO: bounding box based pack selection
+      preferredPack = self.get('preferredMonavDataPack', None)
+      if preferredPack in dataPacks:
+        packName = preferredPack
+      else:
+        # just take the first (and possibly only) pack
+        packName = sorted(dataPacks)[0]
+      mainMonavFolder = self.modrana.paths.getMonavDataPath()
+      monavDataFolder = os.path.abspath(os.path.join(mainMonavFolder, packName, subFolder))
+      print('Monav data folder:\n%s' % monavDataFolder)
+      print(os.path.exists(monavDataFolder))
       try:
         # is Monav initialized ?
         if self.monav is None:
-          # start Monav
+          # start Monav #
+          # only import Monav & company when actually needed
+          # -> the protobuf modules are quite large
           import monav_support
-          self.monav = monav_support.Monav()
+          self.monav = monav_support.Monav(self.modrana.paths.getMonavServerBinaryPath())
           self.monav.startServer()
-        route = self.monav.monavDirections(monavDataFolder, waypoints)
-        return route
-
+        result = self.monav.monavDirections(monavDataFolder, waypoints)
       except Exception, e:
         print('route: Monav route lookup failed')
         print(e)
-        return None
+        traceback.print_exc(file=sys.stdout) # find what went wrong
+        return None, None
+      if result is None:
+        return result, None
+      elif result.type == result.SUCCESS:
+        return result, ROUTING_SUCCESS
+      elif result.type == result.LOAD_FAILED:
+        return result, ROUTING_LOAD_FAILED
+      elif result.type == result.LOOKUP_FAILED:
+        return result, ROUTING_LOOKUP_FAILED
+      elif result.type == result.ROUTE_FAILED:
+        return result, ROUTING_ROUTE_FAILED
+      else:
+        return result, None
     else:
       print("route: no Monav routing data - can't route")
-      return None
+      return None, ROUTING_NO_DATA
+
+  def getAvailableMonavDataPacks(self):
+    """return all available Monav data packs in the main monav data folder"""
+    # basically just list all directories in the Monav data folder
+    mainMonavFolder = self.modrana.paths.getMonavDataPath()
+    dataPacks = os.listdir(mainMonavFolder)
+    dataPacks = filter(lambda x: os.path.isdir(os.path.join(mainMonavFolder, x)), dataPacks )
+    return sorted(dataPacks)
 
   def doAddressRoute(self, start, destination):
     """Route from one point to another, and set that as the active route"""
@@ -376,6 +439,9 @@ class route(ranaModule):
           # create the directions Way object
           self.duration = directions['Directions']['Duration']['html']
           dirs = way.fromGoogleDirectionsResult(directions)
+          #TODO: use seconds from Way object directly
+          #(needs seconds to human representation conversion)
+          #self.duration = dirs.getDuration()
           self.processAndSaveResults(dirs, start, destination, routeRequestSentTimestamp)
       elif key == "onlineRouteAddress2Address":
         (directions, start, destination, routeRequestSentTimestamp) = resultsTuple
@@ -383,6 +449,10 @@ class route(ranaModule):
         self.text = None
         if directions: # is there actually something in the directions ?
           self.duration = directions['Directions']['Duration']['html']
+          #TODO: use seconds from Way object directly
+          #(needs seconds to human representation conversion)
+          #self.duration = dirs.getDuration()
+
           # create the directions Way object
           dirs = way.fromGoogleDirectionsResult(directions)
           self.processAndSaveResults(dirs, start, destination, routeRequestSentTimestamp)
@@ -392,10 +462,25 @@ class route(ranaModule):
         self.sendMessage('ms:turnByTurn:start:%s' % autostart)
       self.set('needRedraw', True)
     elif key == "MonavRoute":
-      (directions, start, destination, routeRequestSentTimestamp) = resultsTuple
-      self.duration = "" # TODO : correct duration
-      dirs = way.fromMonavResult(directions)
-      self.processAndSaveResults(dirs, start, destination, routeRequestSentTimestamp)
+      (result, start, destination, routeRequestSentTimestamp) = resultsTuple
+      directions, returnCode = result
+
+      if returnCode == ROUTING_SUCCESS:
+        self.duration = "" # TODO : correct predicted route duration
+        dirs = way.fromMonavResult(directions)
+        self.processAndSaveResults(dirs, start, destination, routeRequestSentTimestamp)
+      else: # routing failed
+        # show what & why failed
+        if returnCode == ROUTING_LOOKUP_FAILED:
+          self.notify('no ways near start or destination', 3000)
+        elif returnCode == ROUTING_NO_DATA:
+          self.notify('no routing data available', 3000)
+        elif returnCode == ROUTING_LOAD_FAILED:
+          self.notify('failed to load routing data', 3000)
+        elif returnCode == ROUTING_ROUTE_FAILED:
+          self.notify('failed to compute route', 3000)
+        else:
+          self.notify('offline routing failed', 5000)
     elif key == "startAddress":
       self.startAddress = resultsTuple
       self.text = None # clear route detail cache
@@ -793,17 +878,24 @@ class route(ranaModule):
       if not self.directions:
         text = "There is currently no active route."
       elif self.text is None: # the new text for the info-box only once
+        # check for online status
+        online = (self.modrana.dmod.getInternetConnectivityStatus() in (True, None))
         # check if start and destination geocoding is needed
         if not self.routeDetailGeocodingTriggered:
-          self._geocodeStartAndDestination()
-          self.routeDetailGeocodingTriggered = True
+          if online:
+            self._geocodeStartAndDestination()
+            self.routeDetailGeocodingTriggered = True
 
         if self.duration:
           duration = self.duration # a string describing the estimated time to finish the route
         else:
-          duration = "unknown"
+          duration = "? minutes"
         units = self.m.get('units', None) # get the correct units
-        distance = units.m2CurrentUnitString(self.directions.getLength())
+        length = self.directions.getLength()
+        if length:
+          distance = units.m2CurrentUnitString(length)
+        else:
+          distance = "? km"
         steps = self.directions.getMessagePointCount() # number of steps
 
         if self.startAddress:
